==> org.eclipse.papyrus.designer.transformation.languages.cpp.library.source_2.0.0.202105090719.jar: org/eclipse/papyrus/designer/transformation/languages/cpp/library/statemachine/ChangeEventTransformation.java <==
/**
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 */
package org.eclipse.papyrus.designer.transformation.languages.cpp.library.statemachine;

import org.eclipse.papyrus.designer.languages.cpp.profile.C_Cpp.Array;
import org.eclipse.papyrus.designer.transformation.languages.cpp.library.IncludeUtils;
import org.eclipse.papyrus.designer.transformation.languages.cpp.library.statemachine.CDefinitions;
import org.eclipse.papyrus.designer.transformation.languages.cpp.library.statemachine.PThreadTypes;
import org.eclipse.papyrus.designer.transformation.languages.cpp.library.statemachine.SM2ClassesTransformationCore;
import org.eclipse.papyrus.designer.transformation.library.statemachine.SMCodeGeneratorConstants;
import org.eclipse.papyrus.designer.transformation.library.statemachine.TransformationUtil;
import org.eclipse.papyrus.uml.tools.utils.StereotypeUtil;
import org.eclipse.uml2.uml.ChangeEvent;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.util.UMLUtil;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

@SuppressWarnings("all")
public class ChangeEventTransformation {
  @Extension
  protected CDefinitions cdefs;
  
  private SM2ClassesTransformationCore core;
  
  private PThreadTypes ptTypes;
  
  private org.eclipse.uml2.uml.Class superContext;
  
  public ChangeEventTransformation(final SM2ClassesTransformationCore core) {
    this.core = core;
    this.superContext = core.superContext;
    this.ptTypes = core.ptTypes;
    this.cdefs = core.cdefs;
  }
  
  public void createChangeEvents() {
    boolean _isEmpty = this.core.changeEvents.isEmpty();
    if (_isEmpty) {
      return;
    }
    Property threads = this.superContext.createOwnedAttribute(SMCodeGeneratorConstants.THREADS_CHANGE_EVENT, this.ptTypes.getPthread());
    StereotypeUtil.apply(threads, Array.class);
    Array _stereotypeApplication = UMLUtil.<Array>getStereotypeApplication(threads, Array.class);
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("[");
    int _size = this.core.changeEvents.size();
    _builder.append(_size);
    _builder.append("]");
    _stereotypeApplication.setDefinition(_builder.toString());
    Property threadStructs = this.superContext.createOwnedAttribute(SMCodeGeneratorConstants.THREAD_STRUCTS_FOR_CHANGEEVENT, this.core.concurrency.threadStructType);
    StereotypeUtil.apply(threadStructs, Array.class);
    Array _stereotypeApplication_1 = UMLUtil.<Array>getStereotypeApplication(threadStructs, Array.class);
    StringConcatenation _builder_1 = new StringConcatenation();
    _builder_1.append("[");
    int _size_1 = this.core.changeEvents.size();
    _builder_1.append(_size_1);
    _builder_1.append("]");
    _stereotypeApplication_1.setDefinition(_builder_1.toString());
    String WAIT_TIME = "10";
    StringConcatenation _builder_2 = new StringConcatenation();
    _builder_2.append("#ifdef WIN32");
    _builder_2.newLine();
    _builder_2.append("#include <windows.h>");
    _builder_2.newLine();
    _builder_2.append("#elif _POSIX_C_SOURCE >= 199309L");
    _builder_2.newLine();
    _builder_2.append("#include <time.h>   // for nanosleep");
    _builder_2.newLine();
    _builder_2.append("#else");
    _builder_2.newLine();
    _builder_2.append("#include <unistd.h> // for usleep");
    _builder_2.newLine();
    _builder_2.append("#endif");
    IncludeUtils.appendIncludeHeader(this.superContext, _builder_2.toString());
    Operation changeEventOp = this.superContext.createOwnedOperation(SMCodeGeneratorConstants.CHANGE_EVENT_LISTEN_FUNCTION, null, null);
    changeEventOp.createOwnedParameter("id", this.core.intType);
    StringConcatenation _builder_3 = new StringConcatenation();
    _builder_3.append("struct timeval tv;");
    _builder_3.newLine();
    _builder_3.append("struct timespec ts;");
    _builder_3.newLine();
    _builder_3.append("bool oldValue;");
    _builder_3.newLine();
    _builder_3.append("bool newValue;");
    _builder_3.newLine();
    _builder_3.append("bool isFirstTime = true;");
    _builder_3.newLine();
    _builder_3.append("bool periodic = ");
    _builder_3.append(WAIT_TIME);
    _builder_3.append("; //ms");
    _builder_3.newLineIfNotEmpty();
    _builder_3.append("while(true) {");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("switch(id) {");
    _builder_3.newLine();
    {
      for(final ChangeEvent ch : this.core.changeEvents) {
        _builder_3.append("\t\t");
        _builder_3.append("case ");
        String _eventID = TransformationUtil.eventID(ch);
        _builder_3.append(_eventID, "\t\t");
        _builder_3.append(":");
        _builder_3.newLineIfNotEmpty();
        _builder_3.append("\t\t");
        _builder_3.append("\t");
        _builder_3.append("newValue = ");
        String _changeEventExpression = this.getChangeEventExpression(ch);
        _builder_3.append(_changeEventExpression, "\t\t\t");
        _builder_3.append(";");
        _builder_3.newLineIfNotEmpty();
        _builder_3.append("\t\t");
        _builder_3.append("\t");
        _builder_3.append("break;");
        _builder_3.newLine();
      }
    }
    _builder_3.append("\t");
    _builder_3.append("}");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("if (isFirstTime) {");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("oldValue = newValue;");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("isFirstTime = false;");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("} else {");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("#ifdef WIN32");
    _builder_3.newLine();
    _builder_3.append("\t\t\t");
    _builder_3.append("Sleep(periodic);");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("#elif _POSIX_C_SOURCE >= 199309L");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("struct timespec ts;");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("ts.tv_sec = periodic / 1000;");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("ts.tv_nsec = (periodic % 1000) * 1000000;");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("nanosleep(&ts, NULL);");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("#else");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("usleep(periodic * 1000);");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("#endif");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("}");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("if (oldValue != newValue) {");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("//the expression is changed, push an event to the queue");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append(SMCodeGeneratorConstants.EVENT_QUEUE, "\t\t");
    _builder_3.append(".push(statemachine::PRIORITY_2, NULL, id, statemachine::CHANGE_EVENT, 0);");
    _builder_3.newLineIfNotEmpty();
    _builder_3.append("\t");
    _builder_3.append("}");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("oldValue = newValue;");
    _builder_3.newLine();
    _builder_3.append("}");
    this.core.createOpaqueBehavior(this.superContext, changeEventOp, _builder_3.toString());
  }
  
  public String getChangeEventExpression(final ChangeEvent e) {
    if (((e.getChangeExpression() == null) || (!(e.getChangeExpression() instanceof OpaqueExpression)))) {
      return "false";
    } else {
      ValueSpecification _changeExpression = e.getChangeExpression();
      return IterableExtensions.<String>head(((OpaqueExpression) _changeExpression).getBodies());
    }
  }
}

==> org.eclipse.papyrus.designer.transformation.languages.cpp.library.source_3.0.0.202304050728.jar: org/eclipse/papyrus/designer/transformation/languages/cpp/library/statemachine/ChangeEventTransformation.java <==
/**
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 */
package org.eclipse.papyrus.designer.transformation.languages.cpp.library.statemachine;

import org.eclipse.papyrus.designer.languages.cpp.profile.C_Cpp.Array;
import org.eclipse.papyrus.designer.transformation.languages.cpp.library.IncludeUtils;
import org.eclipse.papyrus.designer.transformation.library.statemachine.SMCodeGeneratorConstants;
import org.eclipse.papyrus.designer.uml.tools.utils.StateMachineUtils;
import org.eclipse.papyrus.designer.uml.tools.utils.StereotypeUtil;
import org.eclipse.uml2.uml.ChangeEvent;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.util.UMLUtil;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

@SuppressWarnings("all")
public class ChangeEventTransformation {
  @Extension
  protected CDefinitions cdefs;

  private SM2ClassesTransformationCore core;

  private PThreadTypes ptTypes;

  private org.eclipse.uml2.uml.Class superContext;

  public ChangeEventTransformation(final SM2ClassesTransformationCore core) {
    this.core = core;
    this.superContext = core.superContext;
    this.ptTypes = core.ptTypes;
    this.cdefs = core.cdefs;
  }

  public void createChangeEvents() {
    boolean _isEmpty = this.core.changeEvents.isEmpty();
    if (_isEmpty) {
      return;
    }
    Property threads = this.superContext.createOwnedAttribute(SMCodeGeneratorConstants.THREADS_CHANGE_EVENT, this.ptTypes.getPthread());
    StereotypeUtil.apply(threads, Array.class);
    Array _stereotypeApplication = UMLUtil.<Array>getStereotypeApplication(threads, Array.class);
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("[");
    int _size = this.core.changeEvents.size();
    _builder.append(_size);
    _builder.append("]");
    _stereotypeApplication.setDefinition(_builder.toString());
    Property threadStructs = this.superContext.createOwnedAttribute(SMCodeGeneratorConstants.THREAD_STRUCTS_FOR_CHANGEEVENT, this.core.concurrency.threadStructType);
    StereotypeUtil.apply(threadStructs, Array.class);
    Array _stereotypeApplication_1 = UMLUtil.<Array>getStereotypeApplication(threadStructs, Array.class);
    StringConcatenation _builder_1 = new StringConcatenation();
    _builder_1.append("[");
    int _size_1 = this.core.changeEvents.size();
    _builder_1.append(_size_1);
    _builder_1.append("]");
    _stereotypeApplication_1.setDefinition(_builder_1.toString());
    String WAIT_TIME = "10";
    StringConcatenation _builder_2 = new StringConcatenation();
    _builder_2.append("#ifdef WIN32");
    _builder_2.newLine();
    _builder_2.append("#include <windows.h>");
    _builder_2.newLine();
    _builder_2.append("#elif _POSIX_C_SOURCE >= 199309L");
    _builder_2.newLine();
    _builder_2.append("#include <time.h>   // for nanosleep");
    _builder_2.newLine();
    _builder_2.append("#else");
    _builder_2.newLine();
    _builder_2.append("#include <unistd.h> // for usleep");
    _builder_2.newLine();
    _builder_2.append("#endif");
    IncludeUtils.appendIncludeHeader(this.superContext, _builder_2.toString());
    Operation changeEventOp = this.superContext.createOwnedOperation(SMCodeGeneratorConstants.CHANGE_EVENT_LISTEN_FUNCTION, null, null);
    changeEventOp.createOwnedParameter("id", this.core.intType);
    StringConcatenation _builder_3 = new StringConcatenation();
    _builder_3.append("struct timeval tv;");
    _builder_3.newLine();
    _builder_3.append("struct timespec ts;");
    _builder_3.newLine();
    _builder_3.append("bool oldValue;");
    _builder_3.newLine();
    _builder_3.append("bool newValue;");
    _builder_3.newLine();
    _builder_3.append("bool isFirstTime = true;");
    _builder_3.newLine();
    _builder_3.append("bool periodic = ");
    _builder_3.append(WAIT_TIME);
    _builder_3.append("; //ms");
    _builder_3.newLineIfNotEmpty();
    _builder_3.append("while(true) {");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("switch(id) {");
    _builder_3.newLine();
    {
      for(final ChangeEvent ch : this.core.changeEvents) {
        _builder_3.append("\t\t");
        _builder_3.append("case ");
        String _eventID = StateMachineUtils.eventID(ch);
        _builder_3.append(_eventID, "\t\t");
        _builder_3.append(":");
        _builder_3.newLineIfNotEmpty();
        _builder_3.append("\t\t");
        _builder_3.append("\t");
        _builder_3.append("newValue = ");
        String _changeEventExpression = this.getChangeEventExpression(ch);
        _builder_3.append(_changeEventExpression, "\t\t\t");
        _builder_3.append(";");
        _builder_3.newLineIfNotEmpty();
        _builder_3.append("\t\t");
        _builder_3.append("\t");
        _builder_3.append("break;");
        _builder_3.newLine();
      }
    }
    _builder_3.append("\t");
    _builder_3.append("}");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("if (isFirstTime) {");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("oldValue = newValue;");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("isFirstTime = false;");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("} else {");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("#ifdef WIN32");
    _builder_3.newLine();
    _builder_3.append("\t\t\t");
    _builder_3.append("Sleep(periodic);");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("#elif _POSIX_C_SOURCE >= 199309L");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("struct timespec ts;");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("ts.tv_sec = periodic / 1000;");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("ts.tv_nsec = (periodic % 1000) * 1000000;");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("nanosleep(&ts, NULL);");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("#else");
    _builder_3.newLine();
    _builder_3.append("\t\t    ");
    _builder_3.append("usleep(periodic * 1000);");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("#endif");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("}");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("if (oldValue != newValue) {");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append("//the expression is changed, push an event to the queue");
    _builder_3.newLine();
    _builder_3.append("\t\t");
    _builder_3.append(SMCodeGeneratorConstants.EVENT_QUEUE, "\t\t");
    _builder_3.append(".push(statemachine::PRIORITY_2, NULL, id, statemachine::CHANGE_EVENT, 0);");
    _builder_3.newLineIfNotEmpty();
    _builder_3.append("\t");
    _builder_3.append("}");
    _builder_3.newLine();
    _builder_3.append("\t");
    _builder_3.append("oldValue = newValue;");
    _builder_3.newLine();
    _builder_3.append("}");
    this.core.createOpaqueBehavior(this.superContext, changeEventOp, _builder_3.toString());
  }

  public String getChangeEventExpression(final ChangeEvent e) {
    if (((e.getChangeExpression() == null) || (!(e.getChangeExpression() instanceof OpaqueExpression)))) {
      return "false";
    } else {
      ValueSpecification _changeExpression = e.getChangeExpression();
      return IterableExtensions.<String>head(((OpaqueExpression) _changeExpression).getBodies());
    }
  }
}
